clc;
close all
clear all
format long e

data=dlmread('Datron4000_R6581_UWS_10mV-5.csv',';',1,0);

pkg load optim;

P1=polyfit(data(:,3),data(:,4),1);
Q1=polyval(P1,data(:,3));

i=1;
j=1;
for j=1:(length(data(:,3))/20)
d(j,1)=mean(data(i:i+19,3));
d(j,2)=mean(data(i:i+19,4));
d(j,3)=mean(data(i:i+19,4)-Q1(i:i+19));
i=i+20;
j=j+1;
end;

c=(-11:1e-3:11)';
y1=interp1(d(:,1),d(:,2),c);

% X is the vector of all the values as registered by the 6581 ADC
% beta is a vector of the 16 parameters
% find a value of beta so sum((f(X, beta) - Y)^2) is as small as possible
% Y are the values as measured by the 3458A or whatever else the reference is

beta=[+1.20000000E-06 +1.20000000E-06 +5.00000000E-07 -8.00000000E-07 -2.10000000E-06 +1.00000000E-05 +4.00000000E-05 +3.00000000E-05 +1.00000000E-05 +1.00000000E-05 +2.00000000E-07 -2.00000000E-07 -1.90000000E-07 -4.00000000E-07 -4.00000000E-07 +1.00000170E+00 1 0];

W=pieceWise1(d(:,2), beta);
P2=polyfit(d(:,1),W,1);
Q2=polyval(P2,d(:,1));
disp("Original beta:");
disp("RMSE: "), disp(sqrt(sum((d(:,2)-d(:,1)).^2)));
disp("Max INL: "), disp(max(abs(W-Q2)));
disp("99 percentile INL: "), disp(prctile(abs(W-Q2), 99));
disp("");

X=pieceWise1(y1, beta);
[yfit pfit cvg iter] = leasqr(X, c, beta, "correction4", 1e-6, 100);

C=pieceWise1(d(:,2), beta);

Z=correction4(C, pfit);
P3=polyfit(d(:,1),Z,1);
Q3=polyval(P3,d(:,1));
disp("Fitted beta:");
disp("beta: "), disp(pfit);
disp("RMSE: "), disp(sqrt(sum((Z-d(:,1)).^2)));
disp("Max INL: "), disp(max(abs(Z-Q3)));
disp("99 percentile INL: "), disp(prctile(abs(Z-Q3), 99));
disp("");
% Do this afterwards otherwise it will affect RMSE
pfit(17) = 1;
pfit(18) = 0;

fig1=figure(1,'DefaultAxesFontSize',14);
get(fig1,'position');
get(fig1,'units');
set(fig1,'units','pixels');
set(fig1, 'position',[20 20 1280 961]);
subplot(2,1,1);
title('INL 23\degC @45%rH','FontSize',14);
hold on;
plot(data(:,3),data(:,4)-Q1,'b.');
plot(d(:,1),d(:,3),'g.');
plot(d(:,1),W-Q2,'r.');
plot(d(:,1),Z-Q3,'m.');
grid on;
grid(gca,'minor');
xlabel('K3458A [V]','FontSize',14);
ylabel('INL [V]','FontSize',14);
set(gca, "fontsize", 20);
subplot(2,1,2);
hold on;
plot(data(:,3),data(:,4)-data(:,3),'b.');
plot(d(:,1),d(:,2)-d(:,1),'g.');
plot(d(:,1),W-d(:,1),'r.');
plot(d(:,1),Z-d(:,1),'m.');
grid on;
grid(gca,'minor');
xlabel('K3458A [V]','FontSize',14);
ylabel('Difference to K3458A [V]','FontSize',14);
legend('with original parameter','mean with original parameter','with parameters "turned off"','with leasqr parameters');
set(gca, "fontsize", 20);
return
